(module rsstransforms mzscheme

(require
  (lib "match.ss"))

(provide rsss->rss rss->html rss->txt rss->title rss->summary)

(define rsss->rss
  (match-lambda
    (('rss . contents) `(rss (@ (version "0.91")) ,@(map rsss->rss contents)))
    (('channel . contents) `(channel ,@(map rsss->rss contents)))
    (('qitem title link desc)
      `(item (title ,title) (link ,link) (description ,desc)))
    (all all)))
; final catchall to pass-through unmatched tags -- bad style?

(define rss->html
  (match-lambda
    (('rss ('channel . contents))
     `(top ns pi
        (html
          (head
            (title ,(cadr (assoc 'title contents)))
            (meta (@ (name "description") (contents ,(cadr (assoc 'description contents)))))
            (meta (@ (name "author") (contents ,(cadr (assoc 'author contents)))))
            (meta (@ (name "generator") (contents "Converted from an RSS feed")))
            (meta (@ (name "date") (contents ,(cadr (assoc 'date contents))))))
          (body ""
            (p ,(cadr (assoc 'description contents)))
            (dl
                ,@(map rss->html (list-tail contents 6)))))))
    (('qitem title link desc)
      (rss->html `(item (title ,title) (link ,link) (description ,desc))))
    (('item ('title title) ('link link) ('description desc))
     `(div (dt (a ((href ,link)) ,title)) (dd ,desc)))))

(define rss->txt
  (match-lambda
    (('rss contents) (rss->txt contents))
    (('channel . contents) (apply string-append (map rss->txt contents)))
    (('title title) (format "\n~A\n\n" title))
    (('link link) (format "URL: ~A\n\n" link))
    (('description desc) (format "\t~A\n" desc))
    (('author auth) (format "\t(Compiled by: ~A)\n" auth))
    (('date dated) (format "\t(Date: ~A)\n" dated))
    (('qitem title link desc)
      (rss->txt `(item (title ,title) (link ,link) (description ,desc))))
    (('item ('title title) ('link link) ('description desc))
      (format
        "\n\n~A\n\n\t~A\n\nURL: ~A\n"
        title desc link))
    (all "")))
; final catchall to pass-through unmatched tags -- bad style?

(define rss->summary
  (match-lambda
    (('rss contents) (rss->summary contents))
    (('channel . contents) (apply string-append (map rss->summary contents)))
    (('qitem title link desc)
      (rss->summary `(item (title ,title) (link ,link) (description ,desc))))
    (('item ('title title) ('link link) ('description desc))
      (string-append title "; "))
    (all "")))

(define rss->title
  (match-lambda
    (('rss c) (rss->title c))
    (('channel . c) (apply string-append (map rss->title c)))
    (('title title) title)
    (all "")))

)