(module templater mzscheme

(require (lib "pregexp.ss") "newsfeeds.ss" "styles.ss")

(provide
  get-template
  template-substitute)

(define (template-name servername pathstring default)
  (let ((pathelems (pregexp-split "/" pathstring)))
    (string-append
      "templates/"
      servername
      "/"
      (if (and (list? pathelems) (> (length pathelems) 1))
          (cadr pathelems)
          default)
      ".html")))

(define (get-template . args)
  (call-with-input-file (apply template-name args)
    (lambda (p) (read-string 65535 p))))

(define (template-substitute template tokens)
  (let* ((tl (string-length template))
         (re (pregexp "<\\?se3 .*? \\?>"))
         (feednames (get-feed-names template))
         (feeds (map load-feed feednames))
         (tokens (append tokens (map
                                  (lambda (n f)
                                    (list n (xexpr->string f)))
                                  feednames feeds))))
    (let proc ((tag (pregexp-match-positions re template)) (buf '()) (lastend 0))
      (if (not tag)
          (apply string-append (reverse (cons (substring template lastend tl) buf)))
          (let* ((start (caar tag))
                 (end (cdar tag))
                 (tagname (substring template (+ start 6) (- end 3))))
            (proc
              (pregexp-match-positions re template end tl)
              (cons
                (cadr (or (assoc tagname tokens) (list #t (string-append "<!-- " tagname " -->"))))
                (cons (substring template lastend start) buf))
              end))))))

)
